var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
import React, { useState, useEffect, useRef } from 'react';
import isEqual from 'react-fast-compare';
import styles from './styles';
import { rerenderPropsList, ImageMapperDefaultProps, } from './types';
var ImageMapper = function (props) {
    var containerRef = props.containerRef, active = props.active, fillColorProp = props.fillColor, lineWidthProp = props.lineWidth, mapProp = props.map, srcProp = props.src, strokeColorProp = props.strokeColor, natural = props.natural, heightProp = props.height, widthProp = props.width, imageWidthProp = props.imgWidth, areaKeyName = props.areaKeyName, stayHighlighted = props.stayHighlighted, stayMultiHighlighted = props.stayMultiHighlighted, toggleHighlighted = props.toggleHighlighted, parentWidth = props.parentWidth, responsive = props.responsive;
    var _a = useState(JSON.parse(JSON.stringify(mapProp))), map = _a[0], setMap = _a[1];
    var storedMap = useState(map)[0];
    var _b = useState(false), isRendered = _b[0], setRendered = _b[1];
    var _c = useState(null), imgRef = _c[0], setImgRef = _c[1];
    var _d = useState(false), isClearFnCalled = _d[0], setClearFnCall = _d[1];
    var container = useRef(null);
    var img = useRef(null);
    var canvas = useRef(null);
    var ctx = useRef(null);
    var isInitialMount = useRef(true);
    useEffect(function () {
        if (isInitialMount.current) {
            isInitialMount.current = false;
        }
        else {
            updateCacheMap();
            initCanvas();
            updateCanvas();
        }
    }, [props, isInitialMount]);
    useEffect(function () {
        ctx.current = canvas.current.getContext('2d');
        updateCacheMap();
        setRendered(true);
    }, []);
    useEffect(function () {
        container.current.clearHighlightedArea = function () {
            setMap(storedMap);
            setClearFnCall(true);
        };
        if (containerRef) {
            containerRef.current = container.current;
        }
    }, []);
    useEffect(function () {
        if (isClearFnCalled) {
            setClearFnCall(false);
            initCanvas();
        }
    }, [isClearFnCalled]);
    useEffect(function () {
        if (responsive)
            initCanvas();
    }, [parentWidth]);
    var updateCacheMap = function () {
        setMap(JSON.parse(JSON.stringify(mapProp)));
    };
    var callingFn = function (shape, coords, fillColor, lineWidth, strokeColor) {
        if (shape === 'rect') {
            return drawRect(coords, fillColor, lineWidth, strokeColor);
        }
        if (shape === 'circle') {
            return drawCircle(coords, fillColor, lineWidth, strokeColor);
        }
        if (shape === 'poly') {
            return drawPoly(coords, fillColor, lineWidth, strokeColor);
        }
        return false;
    };
    var drawRect = function (coords, fillColor, lineWidth, strokeColor) {
        var left = coords[0], top = coords[1], right = coords[2], bot = coords[3];
        ctx.current.fillStyle = fillColor;
        ctx.current.lineWidth = lineWidth;
        ctx.current.strokeStyle = strokeColor;
        ctx.current.strokeRect(left, top, right - left, bot - top);
        ctx.current.fillRect(left, top, right - left, bot - top);
    };
    var drawCircle = function (coords, fillColor, lineWidth, strokeColor) {
        ctx.current.fillStyle = fillColor;
        ctx.current.beginPath();
        ctx.current.lineWidth = lineWidth;
        ctx.current.strokeStyle = strokeColor;
        ctx.current.arc(coords[0], coords[1], coords[2], 0, 2 * Math.PI);
        ctx.current.closePath();
        ctx.current.stroke();
        ctx.current.fill();
    };
    var drawPoly = function (coords, fillColor, lineWidth, strokeColor) {
        var newCoords = coords.reduce(function (a, v, i, s) { return (i % 2 ? a : __spreadArray(__spreadArray([], a), [s.slice(i, i + 2)])); }, []);
        // const first = newCoords.unshift();
        ctx.current.fillStyle = fillColor;
        ctx.current.beginPath();
        ctx.current.lineWidth = lineWidth;
        ctx.current.strokeStyle = strokeColor;
        // ctx.current.moveTo(first[0], first[1]);
        newCoords.forEach(function (c) { return ctx.current.lineTo(c[0], c[1]); });
        ctx.current.closePath();
        ctx.current.stroke();
        ctx.current.fill();
    };
    var getDimensions = function (type) {
        if (typeof props[type] === 'function') {
            // @ts-ignore
            return props[type](img.current);
        }
        return props[type];
    };
    var getValues = function (type, measure, name) {
        if (name === void 0) { name = 'area'; }
        var _a = img.current, naturalWidth = _a.naturalWidth, naturalHeight = _a.naturalHeight, clientWidth = _a.clientWidth, clientHeight = _a.clientHeight;
        if (type === 'width') {
            if (responsive)
                return parentWidth;
            if (natural)
                return naturalWidth;
            if (widthProp || name === 'image')
                return measure;
            return clientWidth;
        }
        if (type === 'height') {
            if (responsive)
                return clientHeight;
            if (natural)
                return naturalHeight;
            if (heightProp || name === 'image')
                return measure;
            return clientHeight;
        }
        return 0;
    };
    var initCanvas = function (firstLoad) {
        if (firstLoad === void 0) { firstLoad = false; }
        var imgWidth = getDimensions('width');
        var imgHeight = getDimensions('height');
        var imageWidth = getValues('width', imgWidth);
        var imageHeight = getValues('height', imgHeight);
        if (widthProp || responsive) {
            img.current.width = getValues('width', imgWidth, 'image');
        }
        if (heightProp || responsive) {
            img.current.height = getValues('height', imgHeight, 'image');
        }
        canvas.current.width = imageWidth;
        canvas.current.height = imageHeight;
        container.current.style.width = imageWidth + "px";
        container.current.style.height = imageHeight + "px";
        ctx.current = canvas.current.getContext('2d');
        ctx.current.fillStyle = fillColorProp;
        //ctx.strokeStyle = props.strokeColor;
        if (props.onLoad && firstLoad) {
            props.onLoad(img.current, {
                width: imageWidth,
                height: imageHeight,
            });
        }
        setImgRef(img.current);
        renderPrefilledAreas();
    };
    var hoverOn = function (area, index, event) {
        var shape = area.shape, scaledCoords = area.scaledCoords, fillColor = area.fillColor, lineWidth = area.lineWidth, strokeColor = area.strokeColor;
        if (active) {
            callingFn(shape, scaledCoords, fillColor || fillColorProp, lineWidth || lineWidthProp, strokeColor || strokeColorProp);
        }
        if (props.onMouseEnter)
            props.onMouseEnter(area, index, event);
    };
    var hoverOff = function (area, index, event) {
        if (active) {
            ctx.current.clearRect(0, 0, canvas.current.width, canvas.current.height);
            renderPrefilledAreas();
        }
        if (props.onMouseLeave)
            props.onMouseLeave(area, index, event);
    };
    var click = function (area, index, event) {
        if (stayHighlighted || stayMultiHighlighted || toggleHighlighted) {
            var newArea_1 = __assign({}, area);
            var chosenArea = stayMultiHighlighted ? map : storedMap;
            if (toggleHighlighted && newArea_1.preFillColor) {
                delete newArea_1.preFillColor;
            }
            else if (stayHighlighted || stayMultiHighlighted) {
                newArea_1.preFillColor = area.fillColor || fillColorProp;
            }
            var updatedAreas_1 = chosenArea.areas.map(function (cur) {
                return cur[areaKeyName] === area[areaKeyName] ? newArea_1 : cur;
            });
            setMap(function (prev) { return (__assign(__assign({}, prev), { areas: updatedAreas_1 })); });
        }
        if (props.onClick) {
            event.preventDefault();
            props.onClick(area, index, event);
        }
    };
    var imageClick = function (event) {
        if (props.onImageClick) {
            event.preventDefault();
            props.onImageClick(event);
        }
    };
    var mouseMove = function (area, index, event) {
        if (props.onMouseMove)
            props.onMouseMove(area, index, event);
    };
    var imageMouseMove = function (event) {
        if (props.onImageMouseMove)
            props.onImageMouseMove(event);
    };
    var scaleCoords = function (coords) {
        var scale = widthProp && imageWidthProp && imageWidthProp > 0
            ? widthProp / imageWidthProp
            : 1;
        if (responsive && parentWidth) {
            return coords.map(function (coord) { return coord / (imgRef.naturalWidth / parentWidth); });
        }
        return coords.map(function (coord) { return coord * scale; });
    };
    var renderPrefilledAreas = function () {
        map.areas.map(function (area) {
            if (!area.preFillColor)
                return false;
            callingFn(area.shape, scaleCoords(area.coords), area.preFillColor, area.lineWidth || lineWidthProp, area.strokeColor || strokeColorProp);
            return true;
        });
    };
    var computeCenter = function (area) {
        if (!area)
            return [0, 0];
        var scaledCoords = scaleCoords(area.coords);
        switch (area.shape) {
            case 'circle':
                return [scaledCoords[0], scaledCoords[1]];
            case 'poly':
            case 'rect':
            default: {
                var n_1 = scaledCoords.length / 2;
                var _a = scaledCoords.reduce(function (_a, val, idx) {
                    var y = _a.y, x = _a.x;
                    return (!(idx % 2) ? { y: y, x: x + val / n_1 } : { y: y + val / n_1, x: x });
                }, { y: 0, x: 0 }), scaleY = _a.y, scaleX = _a.x;
                return [scaleX, scaleY];
            }
        }
    };
    var updateCanvas = function () {
        ctx.current.clearRect(0, 0, canvas.current.width, canvas.current.height);
        mapProp.areas.map(function (cur) {
            if (cur.preFillColor) {
                var scaledCoords = scaleCoords(cur.coords);
                hoverOn(__assign(__assign({}, cur), { scaledCoords: scaledCoords }));
                return true;
            }
            return false;
        });
    };
    var renderAreas = function () {
        return map.areas.map(function (area, index) {
            var scaledCoords = scaleCoords(area.coords);
            var center = computeCenter(area);
            var extendedArea = __assign(__assign({}, area), { scaledCoords: scaledCoords, center: center });
            return (React.createElement("area", { key: area[areaKeyName] || index.toString(), shape: area.shape, coords: scaledCoords.join(','), onMouseEnter: function (event) { return hoverOn(extendedArea, index, event); }, onMouseLeave: function (event) { return hoverOff(extendedArea, index, event); }, onMouseMove: function (event) { return mouseMove(extendedArea, index, event); }, onClick: function (event) { return click(extendedArea, index, event); }, href: area.href, alt: "map" }));
        });
    };
    return (React.createElement("div", { id: "img-mapper", style: styles(props).container, ref: container },
        React.createElement("img", { role: "presentation", className: "img-mapper-img", style: styles(props).img, src: srcProp, useMap: "#" + map.name, alt: "map", ref: img, onLoad: function () { return initCanvas(true); }, onClick: imageClick, onMouseMove: imageMouseMove }),
        React.createElement("canvas", { className: "img-mapper-canvas", ref: canvas, style: styles().canvas }),
        React.createElement("map", { className: "img-mapper-map", name: map.name, style: styles().map }, isRendered && renderAreas())));
};
ImageMapper.defaultProps = ImageMapperDefaultProps;
export default React.memo(ImageMapper, function (prevProps, nextProps) {
    var watchedProps = __spreadArray(__spreadArray([], rerenderPropsList), nextProps.rerenderProps);
    var propChanged = watchedProps.some(function (prop) { return prevProps[prop] !== nextProps[prop]; });
    return isEqual(prevProps.map, nextProps.map) && !propChanged;
});
